Package: odbc
Title: Connect to ODBC Compatible Databases (using the DBI Interface)
Version: 1.7.0
Authors@R: c(
    person("Jim", "Hester", role = "aut"),
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Oliver", "Gjoneski", role = "aut"),
    person("Simon", "Couch", email = "simon.couch@posit.co", role = "aut"),
    person("lexicalunit", role = "cph",
           comment = "nanodbc library"),
    person("Google Inc.", role = "cph",
           comment = "cctz library"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: A DBI-compatible interface to ODBC databases.
License: MIT + file LICENSE
URL: https://odbc.r-dbi.org, https://github.com/r-dbi/odbc,
        https://solutions.posit.co/connections/db/
BugReports: https://github.com/r-dbi/odbc/issues
Depends: R (>= 4.0.0)
Imports: bit64, blob (>= 1.2.0), cli, DBI (>= 1.0.0), hms, lifecycle,
        methods, Rcpp (>= 0.12.11), rlang (>= 1.1.0), vctrs
Suggests: connectcreds (>= 0.2.0), covr, DBItest, httr2, knitr,
        magrittr, paws.common, rmarkdown, RSQLite, snowflakeauth,
        testthat (>= 3.0.0), tibble, withr
LinkingTo: Rcpp
ByteCompile: true
Config/Needs/check: pkgbuild
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: GNU make, An ODBC3 driver manager and drivers.
Collate: 'RcppExports.R' 'aaa-odbc-data-type.R' 'connection-pane.R'
        'dbi-connection.R' 'odbc-connection.R' 'db.R' 'dbi-driver.R'
        'dbi-result.R' 'dbi-table.R' 'dbi.R' 'driver-access.R'
        'driver-bigquery.R' 'driver-databricks.R' 'driver-db2.R'
        'driver-hana.R' 'driver-hive.R' 'driver-impala.R'
        'driver-mysql.R' 'driver-netezza.R' 'driver-oracle.R'
        'driver-postgres.R' 'driver-redshift.R' 'driver-snowflake.R'
        'driver-spark.R' 'driver-sql-server.R' 'driver-sqlite.R'
        'driver-teradata.R' 'driver-vertica.R'
        'import-standalone-obj-type.R'
        'import-standalone-types-check.R' 'odbc-config.R'
        'odbc-data-sources.R' 'odbc-drivers.R' 'odbc-package.R'
        'odbc.R' 'utils.R' 'zzz.R'
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-05-05 18:45:03 UTC; simoncouch
Author: Jim Hester [aut],
  Hadley Wickham [aut, cre],
  Oliver Gjoneski [aut],
  Simon Couch [aut],
  lexicalunit [cph] (nanodbc library),
  Google Inc. [cph] (cctz library),
  Posit Software, PBC [cph, fnd]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-05-09 15:00:02 UTC
