# Generated by roxygen2: do not edit by hand

S3method(odbcListColumns,OdbcConnection)
S3method(odbcListObjectTypes,default)
S3method(odbcListObjects,OdbcConnection)
S3method(odbcPreviewObject,OdbcConnection)
export(databricks)
export(isTempTable)
export(odbc)
export(odbcDataType)
export(odbcEditDrivers)
export(odbcEditSystemDSN)
export(odbcEditUserDSN)
export(odbcListColumns)
export(odbcListConfig)
export(odbcListDataSources)
export(odbcListDrivers)
export(odbcListObjectTypes)
export(odbcListObjects)
export(odbcPreviewObject)
export(odbcSetTransactionIsolationLevel)
export(quote_value)
export(redshift)
export(snowflake)
exportClasses("DB2/AIX64")
exportClasses("Microsoft SQL Server")
exportClasses("Spark SQL")
exportClasses("Vertica Database")
exportClasses(ACCESS)
exportClasses(BigQuery)
exportClasses(DatabricksOdbcDriver)
exportClasses(HDB)
exportClasses(Hive)
exportClasses(Impala)
exportClasses(MySQL)
exportClasses(NetezzaSQL)
exportClasses(OdbcConnection)
exportClasses(OdbcDriver)
exportClasses(OdbcResult)
exportClasses(Oracle)
exportClasses(PostgreSQL)
exportClasses(Redshift)
exportClasses(RedshiftOdbcDriver)
exportClasses(SQLite)
exportClasses(Snowflake)
exportClasses(SnowflakeOdbcDriver)
exportClasses(Teradata)
exportMethods(dbAppendTable)
exportMethods(dbBegin)
exportMethods(dbBind)
exportMethods(dbClearResult)
exportMethods(dbColumnInfo)
exportMethods(dbCommit)
exportMethods(dbConnect)
exportMethods(dbDataType)
exportMethods(dbDisconnect)
exportMethods(dbExecute)
exportMethods(dbExistsTable)
exportMethods(dbFetch)
exportMethods(dbGetInfo)
exportMethods(dbGetQuery)
exportMethods(dbGetRowCount)
exportMethods(dbGetRowsAffected)
exportMethods(dbGetStatement)
exportMethods(dbHasCompleted)
exportMethods(dbIsValid)
exportMethods(dbListFields)
exportMethods(dbListTables)
exportMethods(dbQuoteIdentifier)
exportMethods(dbQuoteString)
exportMethods(dbRemoveTable)
exportMethods(dbRollback)
exportMethods(dbSendQuery)
exportMethods(dbSendStatement)
exportMethods(dbWriteTable)
exportMethods(odbcDataType)
exportMethods(show)
exportMethods(sqlCreateTable)
exportMethods(sqlData)
import(DBI)
import(methods)
import(rlang)
importFrom(Rcpp,sourceCpp)
importFrom(bit64,integer64)
importFrom(blob,blob)
importFrom(hms,hms)
importFrom(lifecycle,deprecated)
importFrom(utils,packageVersion)
useDynLib(odbc, .registration = TRUE)
