# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

list_drivers_ <- function() {
    .Call(`_odbc_list_drivers_`)
}

list_data_sources_ <- function() {
    .Call(`_odbc_list_data_sources_`)
}

odbc_connect <- function(connection_string, timezone = "", timezone_out = "", encoding = "", name_encoding = "", bigint = 0L, timeout = 0L, r_attributes = NULL, interruptible_execution = TRUE) {
    .Call(`_odbc_odbc_connect`, connection_string, timezone, timezone_out, encoding, name_encoding, bigint, timeout, r_attributes, interruptible_execution)
}

has_result <- function(p) {
    .Call(`_odbc_has_result`, p)
}

connection_info <- function(p) {
    .Call(`_odbc_connection_info`, p)
}

connection_quote <- function(p) {
    .Call(`_odbc_connection_quote`, p)
}

connection_release <- function(p) {
    invisible(.Call(`_odbc_connection_release`, p))
}

connection_begin <- function(p) {
    invisible(.Call(`_odbc_connection_begin`, p))
}

connection_commit <- function(p) {
    invisible(.Call(`_odbc_connection_commit`, p))
}

connection_rollback <- function(p) {
    invisible(.Call(`_odbc_connection_rollback`, p))
}

connection_valid <- function(p) {
    .Call(`_odbc_connection_valid`, p)
}

connection_sql_tables <- function(p, catalog_name = NULL, schema_name = NULL, table_name = NULL, table_type = NULL) {
    .Call(`_odbc_connection_sql_tables`, p, catalog_name, schema_name, table_name, table_type)
}

connection_sql_catalogs <- function(p) {
    .Call(`_odbc_connection_sql_catalogs`, p)
}

connection_sql_schemas <- function(p) {
    .Call(`_odbc_connection_sql_schemas`, p)
}

connection_sql_table_types <- function(p) {
    .Call(`_odbc_connection_sql_table_types`, p)
}

connection_sql_columns <- function(p, column_name = NULL, catalog_name = NULL, schema_name = NULL, table_name = NULL) {
    .Call(`_odbc_connection_sql_columns`, p, column_name, catalog_name, schema_name, table_name)
}

transactionLevels <- function() {
    .Call(`_odbc_transactionLevels`)
}

set_transaction_isolation <- function(p, level) {
    invisible(.Call(`_odbc_set_transaction_isolation`, p, level))
}

bigint_mappings <- function() {
    .Call(`_odbc_bigint_mappings`)
}

result_release <- function(r) {
    invisible(.Call(`_odbc_result_release`, r))
}

result_active <- function(r) {
    .Call(`_odbc_result_active`, r)
}

result_completed <- function(r) {
    .Call(`_odbc_result_completed`, r)
}

new_result <- function(p, sql, immediate) {
    .Call(`_odbc_new_result`, p, sql, immediate)
}

result_fetch <- function(r, n_max = -1L) {
    .Call(`_odbc_result_fetch`, r, n_max)
}

result_column_info <- function(r) {
    .Call(`_odbc_result_column_info`, r)
}

result_bind <- function(r, params, batch_rows) {
    invisible(.Call(`_odbc_result_bind`, r, params, batch_rows))
}

result_insert_dataframe <- function(r, df, batch_rows) {
    invisible(.Call(`_odbc_result_insert_dataframe`, r, df, batch_rows))
}

result_describe_parameters <- function(r, df) {
    invisible(.Call(`_odbc_result_describe_parameters`, r, df))
}

result_rows_affected <- function(r) {
    .Call(`_odbc_result_rows_affected`, r)
}

result_row_count <- function(r) {
    .Call(`_odbc_result_row_count`, r)
}

column_types <- function(df) {
    invisible(.Call(`_odbc_column_types`, df))
}

