Thanks for using this project.

#### If you have an error or support request

- Read the error message and documentation.
- Search [existing issues](https://github.com/Koka/odbc-rs/issues), [closed issues](https://github.com/Koka/odbc-rs/issues?page=1&state=closed) first.
- If the issue is with a dependency of this project, post on the dependency's repo.
- If you can fix the issue, submit a PR.
- If the issue persists, post on the issue tracker. Include any information that could help others reproduce and fix.

#### If you have a feature proposal or want to contribute

- Post your proposal on the issue tracker so we can review it together. Some proposals may not be a good fit for the project.
- Contribute a PR (don't forget to write all necessary tests, please).
- Respond to code review.
- Watch the PR be merged.
